/* src/welcome.c - Diverses commandes sur le module welcome
 *
 * Copyright (C) 2002-2005 David Cortier  <Cesar@irc.jeux.fr>
 *                         Romain Bignon  <Progs@kouak.org>
 *                         Benjamin Beret <kouak@kouak.org>
 *
 * site web: http://sf.net/projects/scoderz/
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: welcome.c,v 1.25 2005/07/03 22:30:48 romexzf Exp $
 */

#include "main.h"
#ifdef USE_WELCOMESERV
#include "cs_cmds.h"
#include "debug.h"
#include "config.h"
#include "outils.h"
#include "welcome.h"

static struct welcomeinfo *welcomehead = NULL;
static int WelcomeCount = 0;

static void gwelcome_insert(struct welcomeinfo *w)
{
	struct welcomeinfo *tmp = welcomehead;

	if(1 == WelcomeCount || w->level >= tmp->level)
	{
		w->next = tmp;
		welcomehead = w;
	}
	else
	{
		for(;tmp->next && w->level < tmp->next->level; tmp = tmp->next);
		w->next = tmp->next;
		tmp->next = w;
	}
}

static struct welcomeinfo *gwelcome_add(int level, const char *msg)
{
	struct welcomeinfo *new = malloc(sizeof *new);

   	if(!new)
   	{
	   	Debug(W_MAX, "add_loadwelcome, malloc a chou pour le welcomeinfo %s", msg);
	   	return NULL;
   	}

	Strncpy(new->msg, msg, sizeof new->msg - 1);
	new->level = level;
	new->id = ++WelcomeCount;
	new->view = 0;
	gwelcome_insert(new);
	return new;
}

/* GLOBWELCOME ADD <level> <message>
 *             DEL <id>
 *             SET [ON|OFF]
 *             LIST
 *             MOD <id> <level>
 */

int global_welcome(aNick *nick, aChan *chan, int parc, char **parv)
{
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "SET"))
		switch_option(nick, parc > 1 ? parv[2] : NULL, "welcome", "serveur", &ConfFlag, CF_WELCOME);

	else if(!strcasecmp(cmd, "ADD"))
	{
		struct welcomeinfo *w;
		int level;

		if(parc < 3) return csntc(nick, "Syntaxe: %s ADD <level> <message>", parv[0]);

		if(!is_num(parv[2]) || (level = atoi(parv[2])) < 1 || level > 100)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		if((w = gwelcome_add(level, parv2msg(parc, parv, 3, 250))))
		{
			csntc(nick, "Welcome bien ajout au level %d", level);
			csntc(nick, "\2[Message de bienvenue #%d]\2 - %s", w->id, w->msg);
		}
	}
	else if(!strcasecmp(cmd, "DEL"))
	{
		struct welcomeinfo **tmpp = &welcomehead, *tmp;
		int id;

		if(parc < 2) return csntc(nick, "Syntaxe: %s DEL <ID>", parv[0]);

		if(!is_num(parv[2])) return csntc(nick, "L'id doit tre un nombre.");

		for(id = atoi(parv[2]); (tmp = *tmpp); tmpp = &tmp->next)
			if(id == tmp->id)
			{
				*tmpp = tmp->next;
				free(tmp);
				for(--WelcomeCount, tmp = welcomehead; tmp; tmp = tmp->next)
					if(tmp->id >= id) --tmp->id;
				id = -1; /* mark it as found */
				break;
			}

		if(id >= 0)	return csntc(nick, "Aucun message ayant pour id %d", id);

		csntc(nick, "Le message #%s a bien t supprim (%d messages restants)",
			parv[2], WelcomeCount);
	}
	else if(!strcasecmp(cmd, "LIST"))
	{
		struct welcomeinfo *w = welcomehead;

		if(!w) return csntc(nick, "Aucun message de bienvenue.");

		csntc(nick, "\2#Id Level View Message");
		for(;w;w = w->next) csntc(nick, "#%d %d %d %s", w->id, w->level, w->view, w->msg);

		return 0;
	}
	else if(!strcasecmp(cmd, "MOD"))
	{
		int id, level;
		struct welcomeinfo *wh, **tmp = &welcomehead;

		if(parc < 3) return csntc(nick, "Syntax: %s MOD <ID> <level>", parv[0]);

		if(!is_num(parv[2])) return csntc(nick, "Veuillez donner un ID correct");

		if(!is_num(parv[3]) || (level = atoi(parv[3])) < 1 || level > 100)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		for(id = atoi(parv[2]); (wh = *tmp); tmp = &wh->next)
			if(wh->id == id)
			{
				*tmp = wh->next; /* removes it from list */
				wh->level = level; /* updates level */
				gwelcome_insert(wh); /* then finally inserts it at its new offset. */
				break;
			}

		if(!wh) return csntc(nick, "Aucun message ayant pour id %d", id);
		csntc(nick, "Le level du message #%d est maintenant \2%d\2", id, wh->level);
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), cmd);

	write_welcome();
	return 0;
}

void choose_welcome(const char *num)
{
	struct welcomeinfo *tmp = welcomehead, *last = tmp;
	int rnd;

	if(!tmp) return;

	for(rnd = rand() % 100;tmp; tmp = tmp->next)
		if(tmp->level >= rnd)
		{
			last = tmp;
			if((float) tmp->level/rnd > 2) break;
		}
	++last->view;
	putserv("%s %s %s :\2[Message de bienvenue #%d]\2 - %s", ws.num,
		GetConf(CF_PRIVWELCOME) ? TOKEN_PRIVMSG : TOKEN_NOTICE, num, last->id, last->msg);
}

void write_welcome(void)
{
	FILE *fp = fopen(WELCOME_FILE, "w");
	struct welcomeinfo *w = welcomehead;

	if(!fp) return;

	fprintf(fp, "%d\n%s\n%s\n", GetConf(CF_WELCOME), user_motd, admin_motd);
	for(;w;w = w->next) fprintf(fp, "%d :%s\n", w->level, w->msg);
   	fclose(fp);
}

void load_welcome(void)
{
	char buf[420], *tmp;
	int t = 1;
	FILE *fp = fopen(WELCOME_FILE, "r");

	if(!fp) return;

	while(fgets(buf, sizeof buf, fp))
   	{
		strip_newline(buf);
		if(!strncmp(buf, "VERSION", 7)) continue;
		else if(t == 1) { if(atoi(buf)) ConfFlag |= CF_WELCOME; }
		else if(t == 2) Strncpy(user_motd, buf, 390);
		else if(t == 3) Strncpy(admin_motd, buf, 390);
		else if((tmp = strchr(buf, ' ')))
		{
			*tmp++ = 0;
			gwelcome_add(atoi(buf), ++tmp);
		}
		t++;
	}
	fclose(fp);
}

#endif
